package com.fr.plugin.decision.request.handler;

import com.finebi.activate.foundation.StableManager;
import com.finebi.burger.api.bean.configuration.table.field.FineBusinessField;
import com.finebi.burger.api.bean.configuration.table.page.ConfLimitInfo;
import com.finebi.burger.api.bean.configuration.table.page.ConfPageInfo;
import com.finebi.burger.api.request.configuration.table.DataBaseAddTableItemRequestBean;
import com.finebi.burger.api.request.configuration.table.DataBaseAddTableRequestBean;
import com.finebi.burger.api.response.configuration.table.TableAddResponseBean;
import com.finebi.common.api.vo.pack.FineBusinessPackage;
import com.finebi.common.api.vo.relation.FineBusinessRelation;
import com.finebi.common.api.vo.table.FineDBBusinessTable;
import com.finebi.direct.burger.api.service.FineTableService;
import com.finebi.direct.burger.api.service.FineUserInfoService;
import com.finebi.common.api.cell.user.authority.UserAuthority;
import com.finebi.direct.common.impl.engine.BurgerTableDataCellImpl;
import com.finebi.foundation.api.reponse.FineRespond;
import com.finebi.foundation.api.service.FineService;
import com.finebi.foundation.api.service.context.BaseContext;
import com.finebi.foundation.api.structure.result.BIDetailResult;
import com.finebi.utils.CompareUtils;
import com.fr.decision.webservice.v10.user.UserService;
import com.fr.json.JSONArray;
import com.fr.json.JSONObject;
import com.fr.log.FineLoggerFactory;
import com.fr.plugin.decision.HWUtils;
import com.fr.plugin.decision.core.ActionFactory;
import com.fr.plugin.decision.core.ExcelDmlBean;
import com.fr.plugin.decision.core.action.ExcelDmlAction;
import com.fr.plugin.decision.dao.ExcelLinkService;
import com.fr.plugin.decision.utils.JdbcUtils;
import com.fr.stable.StringUtils;
import com.fr.third.fasterxml.jackson.core.JsonGenerationException;
import com.fr.third.fasterxml.jackson.databind.JsonMappingException;
import com.fr.third.fasterxml.jackson.databind.ObjectMapper;
import com.finebi.direct.common.api.cell.context.CellCreator;
import com.finebi.common.api.vo.table.FineBusinessTable;
import com.finebi.common.api.vo.table.FineExcelBusinessTable;
import com.fr.web.utils.WebUtils;

import javax.servlet.http.HttpServletRequest;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.sql.Connection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;

public class UpgradeHandler extends HWAbstractHandler {
    public static final String PATH="/excelmng/upgrade";
    @Override
    protected void deal(HttpServletRequest req, HttpServletResponse res, JSONObject para, JSONObject result) throws Exception {

        String connection = para.getString("connection");
        UserAuthority authorityUser = getFineUserInfoService().getUserAuthority(req);

        Connection conn = JdbcUtils.getConnection(connection);

        if (conn == null){
            FineLoggerFactory.getLogger().error("HW_excel导入：数据链接 " + connection + " 获取异常" );
            WebUtils.printAsJSON(res, new JSONObject().put("status","fail").put("errorText","获取数据连接异常"));
            return;
        }

        final String createUser = UserService.getInstance().getCurrentUserIdFromCookie(req);

        JSONArray importDbResult = JSONArray.create();
        JSONArray addDbResult = JSONArray.create();
        JSONObject finalResult = JSONObject.create();

        List<FineBusinessTable> allTables = CellCreator.getCellProvider().tableCell().getAllTableWithoutAuth();
        allTables.stream().filter(fineBusinessTable -> fineBusinessTable instanceof FineExcelBusinessTable).limit(15).forEach(fineBusinessTable -> {
            Map<String, List> values = new HashMap<String, List>();
            JSONArray items = JSONArray.create();

            List<FineBusinessField> fields = ((FineExcelBusinessTable) fineBusinessTable).getExcelFields();

            try {
                Optional<BIDetailResult> optionalBIDetailResult = new BurgerTableDataCellImpl().getRealDataWithAuth(fineBusinessTable, new ConfLimitInfo(500000, new ConfPageInfo(500000, 1)), authorityUser);
                BIDetailResult detailResult = optionalBIDetailResult.get();
                values = HWUtils.getValues(fields, detailResult);
                for(int i = 0; i < fields.size(); i++){
                    FineBusinessField field = fields.get(i);
                    JSONObject item = new JSONObject();
                    item.put("colLength", "8000");
                    item.put("colName", field.getName());
                    item.put("colType", this.getFieldSqlType(field.getType()));
                    item.put("excelCol", i);
                    item.put("id", field.getId());
                    items.put(item);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }

            String schema  = "";
            String tableName =  fineBusinessTable.getName();

            ExcelDmlAction excelDmlAction = ActionFactory.getInstance().create("add");

            excelDmlAction.setExcelDmlBean( new ExcelDmlBean(connection, schema, tableName, items, values) );

            try {
                JSONObject dmlResult = excelDmlAction.execute();
                importDbResult.put(dmlResult);

                if (StringUtils.equals("success", dmlResult.getString("status")) && dmlResult.getBoolean("addLink")) {
                    ExcelLinkService.getInstance().addLink(tableName, connection, schema, tableName, createUser);
                    FineLoggerFactory.getLogger().info("HW_excel导入：FineDB 新增数据成功" + tableName);
                }

                if (StringUtils.equals("success", dmlResult.getString("status")) || StringUtils.contains(dmlResult.getString("errorText"), "existed")) {
                    ObjectMapper mapper = new ObjectMapper();
                    JSONObject table = JSONObject.create().put("tableName", fineBusinessTable.getName()).put("connectionName", connection);
                    JSONArray tables = JSONArray.create().put(table);
                    JSONObject tablesBean = JSONObject.create().put("tables", tables);
                    try{
                        Optional<FineBusinessPackage> packageIdWithoutAuth = CellCreator.getCellProvider().packageCell().getPackageByTableNameWithoutAuth(tableName);
                        FineBusinessPackage businessPackage = packageIdWithoutAuth.get();
                        String packId = businessPackage.getId();

                        List<FineBusinessRelation> fineBusinessRelations = CellCreator.getCellProvider().relationPathCell().getRelationsByTableNameWithoutAuth(fineBusinessTable.getName());

                        CellCreator.getCellProvider().tableCell().removeTable(fineBusinessTable.getName(), CellCreator.getCellProvider().userCellProvider().authorityCell().getSuperAuthority());

                        try {
                            String tableString = tablesBean.toString();
                            FineBusinessTable[] businessTables = createBusinessTablesFromDBNewAddTableInfo(mapper.readValue(tableString, DataBaseAddTableRequestBean.class), authorityUser.getUser().getInfo().getUserId());
                            TableAddResponseBean responed = getTableService().addNewTable(packId, authorityUser, businessTables);
                            CellCreator.getCellProvider().relationPathCell().addRelations(fineBusinessRelations, authorityUser);
                            addDbResult.put(FineRespond.success(responed));
                        } catch (JsonMappingException e) {
                            e.printStackTrace();
                        } catch (JsonGenerationException e) {
                            e.printStackTrace();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    } catch(Exception e){
                        e.printStackTrace();
                    }
                }
            } catch(Exception e){
                e.printStackTrace();
            }
        });

        WebUtils.printAsJSON(res, finalResult.put("importDbResult", importDbResult).put("addDbResult", addDbResult));
    }

    @Override
    public String getPath() {
        return PATH;
    }

    protected static FineTableService getTableService() {
        return getBean(FineTableService.class);
    }

    private static <T extends FineService> T getBean(Class<T> annotatedClass) {
        return getContext().getServiceBean(annotatedClass);
    }
    /**
     * service获取集合
     */
    private static BaseContext getContext() {
        return StableManager.getContext();
    }

    private int getFieldSqlType(int fieldType) {
        if (CompareUtils.isEqual(fieldType, 16)) {
            return 12;
        } else if (CompareUtils.isEqual(fieldType, 32)) {
            return 8;
        } else if (CompareUtils.isEqual(fieldType, 48)) {
            return 91;
        }
        return 12;
    }

    public static FineBusinessTable[] createBusinessTablesFromDBNewAddTableInfo(DataBaseAddTableRequestBean bean, String userId) {
        List<DataBaseAddTableItemRequestBean> items = bean.getTables();
        FineBusinessTable[] businessTables = new FineBusinessTable[items.size()];
        for (int i = 0; i < businessTables.length; i++) {
            DataBaseAddTableItemRequestBean item = items.get(i);
            FineBusinessTable table = createBusinessTableByItem(item, userId);
            businessTables[i] = table;
        }
        return businessTables;
    }

    private static FineBusinessTable createBusinessTableByItem(DataBaseAddTableItemRequestBean item, String userId) {

            return createDBBusinessTable(item, userId);
    }

    private static FineBusinessTable createDBBusinessTable(DataBaseAddTableItemRequestBean item, String userId) {
        FineDBBusinessTable table = new FineDBBusinessTable();
        table.setConnectionName(item.getConnectionName());
        table.setTableName(item.getDbTableName());
        table.setCreateByWithId(userId);
        String tableName = getTableNameWithConnection(item);
        table.setId(tableName);
        table.setName(tableName);
        return table;
    }

    private static String getTableNameWithConnection(DataBaseAddTableItemRequestBean itemBean) {
        return itemBean.getDbTableName();
    }

    protected static FineUserInfoService getFineUserInfoService() {
        return getBean(FineUserInfoService.class);
    }
}
