!(function () {
    var UpgradePane = BI.inherit(BI.Widget, {
        props: {
            baseCls: "dec-mng-upgrade"
        },

        _store: function () {
            return BI.Models.getModel("dec.model.mng.upgrade");
        },

        watch: {
            addDbResult: function (v) {
                this.addList.populate(this._createAddItems(v));
            },
            importDbResult: function (v) {
                this.importList.populate(this._createImportItems(v));
            }
        },

        beforeInit: function (callback) {
            this.store.initData(callback);
        },
        
        render: function () {
            var self = this;
            return {
                type: "bi.vertical",
                width: 640,
                vgap: 15,
                items: [{
                    type: "bi.vertical_adapt",
                    items: [{
                        type: "bi.label",
                        textAlign: "right",
                        cls: "dec-font-weight-bold",
                        text: "数据源:",
                        title: "数据源",
                        width: 100
                    }, {
                        el: {
                            type: "bi.text_value_combo",
                            lgap: 4,
                            cls: "bi-border",
                            textAlign: "left",
                            items: this.model.connectionItems,
                            width: 480,
                            height: 22,
                            ref: function (_ref) {
                                self.connection = _ref;
                            }
                        },
                        lgap: 10
                    }]
                }, {
                    type: "bi.center_adapt",
                    items: [{
                        type: "bi.button",
                        width: 200,
                        text: "重名检测",
                        handler: function () {
                            Dec.HW.checkName({
                                connection: self.connection.getValue()[0]
                            }, function (res) {
                                var tip = "以下excel与数据库中已存在的表重名:"
                                BI.each(res.checkResult, function (i, tableName) {
                                    tip += "\n" + tableName;
                                })
                                BI.Msg.toast(tip, {
                                    autoClose: false,
                                    level: "warning"
                                });
                            });
                        }
                    }, {
                        type: "bi.button",
                        width: 200,
                        text: "升级旧Excel",
                        handler: function () {
                            Dec.HW.upgrade({
                                connection: self.connection.getValue()[0]
                            }, function (res) {
                                self.store.setAddDbResult(res.addDbResult);
                                self.store.setImportDbResult(res.importDbResult);
                            });
                        }
                    }]
                }, {
                    type: "bi.vertical",
                    items: [{
                        type: "bi.label",
                        text: "------Excel导入成数据库表报错信息------",
                        textAlign: "left"
                    }, {
                        type: "bi.button_group",
                        height: 300,
                        items: [],
                        layouts: [{
                            type: "bi.vertical",
                            scrolly: true
                        }],
                        ref: function (_ref) {
                            self.importList = _ref;
                        }
                    }, {
                        type: "bi.label",
                        text: "------从数据库表创建到自助数据集报错信息------",
                        textAlign: "left"
                    }, {
                        type: "bi.button_group",
                        height: 300,
                        items: [],
                        layouts: [{
                            type: "bi.vertical",
                            scrolly: true
                        }],
                        ref: function (_ref) {
                            self.addList = _ref;
                        }
                    }]
                }]
            };
        },

        _createAddItems: function (array) {
            var items = BI.filter(array, function (idx, item) {
                return !item.success;
            });
            return BI.map(items, function (idx, item) {
                return {
                    type: "bi.label",
                    textAlign: "left",
                    text: item.errorMsg
                }
            });
        },

        _createImportItems: function (array) {
            return BI.map(array, function (idx, item) {
                return {
                    type: "bi.label",
                    textAlign: "left",
                    text: item.status === "failed" ? item.errorText : item.text
                }
            })
        }
    });
    BI.shortcut("dec.mng.upgrade", UpgradePane);
})();